<?php 

namespace Concrete\Package\GeoContent\Src\Models;

use Concrete\Package\GeoContent\Src\Form\ValidationException;
use Database;
use Doctrine\Common\Collections\ArrayCollection;
use Stack;

/**
 * @Entity
 * @HasLifecycleCallbacks
 * @Table(
 *  name="GeoContentTerritories",
 *  indexes={
 *      @Index(name="territory_priority", columns={"territoryDefault", * "territoryPriority"}),
 *      @Index(name="territory_stack", columns={"stackID"})
 *  }
 * )
 */
class Territory extends BaseEntity
{

    /**
     * @Id
     * @Column(name="territoryID", type="integer")
     * @GeneratedValue
     */
    public $id;

    /** @Column(name="territoryName", length=255) */
    public $name;

    /** @Column(name="stackID", type="integer") */
    public $stackID;

    /** @Column(name="territoryColumn", length=255, nullable=true) */
    public $column;

    /** @Column(name="territoryPriority", type="integer") */
    public $priority;

    /** @Column(name="territoryDefault", type="boolean") */
    public $default;

    /**
     * @OneToMany(targetEntity="TerritoryValue", orphanRemoval=true, mappedBy="territory", cascade={"persist"})
     * @Var ArrayCollection
     */
    public $values;

    /**
     * @OneToMany(targetEntity="TerritoryVariation", orphanRemoval=true, mappedBy="territory", cascade={"persist"})
     * @Var ArrayCollection
     */
    public $variations;

    public function __construct()
    {
        parent::__construct();
        $this->values = new ArrayCollection();
        $this->variations = new ArrayCollection();
    }

    /**
     * Adds a value to a territory
     * @param TerritoryValue $value
     */
    public function addValue(TerritoryValue $value)
    {
        $this->values[] = $value;
        $value->setTerritory($this);
    }

    /**
     * Updates the values of the territory to match those in the array, adding or removing as necessary
     * @param $values
     */
    public function updateValues($values)
    {
        $values = array_map('trim', $values);
        $rawValues = $this->getValuesArray();

        $add = array_diff($values, $rawValues);
        $remove = array_diff($rawValues, $values);

        foreach ($add as $toAdd) {
            $value = new TerritoryValue();
            $value->value = $toAdd;
            $this->addValue($value);
        }

        $em = Database::connection()->getEntityManager();
        $repo = $em->getRepository('Concrete\Package\GeoContent\Src\Models\TerritoryValue');
        foreach ($remove as $toRemove) {
            $value = $repo->findOneBy(array('territory' => $this->id, 'value' => $toRemove));
            $this->values->removeElement($value);
        }
    }

    /**
     * Returns the values as a plain array
     * @return array
     */
    public function getValuesArray()
    {
        $values = array();
        foreach ($this->values as $value) {
            $values[] = $value->value;
        }
        return $values;
    }

    /**
     * Validates the entity
     * @PrePersist
     * @PreUpdate
     */
    public function validate()
    {
        $errors = array();
        if (empty($this->name)) {
            $errors[] = t('A territory name must be provided.');
        }

        $existing = self::findOneBy(array('name' => $this->name));
        if ($existing && $existing->id !== $this->id) {
            $errors[] = t('The territory name must be unique.');
        }

        if (!empty($errors)) {
            throw new ValidationException($errors);
        }
    }

    /**
     * Returns the next priority that should be used when a territory is added
     * @return integer
     */
    public static function getNextPriority()
    {
        $qb = Database::connection()->getEntityManager()->createQueryBuilder();

        return $qb->select($qb->expr()->max('territory.priority'))
            ->from('Concrete\Package\GeoContent\Src\Models\Territory', 'territory')
            ->where($qb->expr()->eq('territory.default', 0))
            ->getQuery()->getSingleScalarResult() + 1;
    }

    /**
     * Deletes the stack associated with a territory when the territory is removed
     * @PreRemove
     */
    public function onRemove()
    {
        Stack::getByID($this->stackID)->delete();
    }

    /**
     * Deletes the related territory variation records
     */
    public function removeTerritoryVariations()
    {
        foreach($this->variations AS $territoryVariation) {
            $territoryVariation->delete();
        }
    }
}